import csv
import psycopg2

from database import db, app
from flask_covid19.blueprints.app_all.all_config import BlueprintConfig
from flask_covid19.blueprints.data_ecdc.ecdc_model import EcdcDateReported
from flask_covid19.blueprints.data_ecdc.ecdc_model_import import EcdcImport


class EcdcServiceImport:
    def __init__(self, database, config: BlueprintConfig):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" ECDC Service Import [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = config
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" ECDC Service Import [ready] ")

    def import_file(self):
        app.logger.info(" import ECDC [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info(" import into TABLE: "+self.cfg.tablename+" from "+self.cfg.cvsfile_path)
        app.logger.info("------------------------------------------------------------")
        k = 0
        try:
            EcdcImport.remove_all()
            with open(self.cfg.cvsfile_path, newline='') as csv_file:
                file_reader = csv.DictReader(csv_file, delimiter=',', quotechar='"')
                for row in file_reader:
                    date_rep = row['dateRep']
                    d = EcdcDateReported.create_new_object_factory(my_date_rep=date_rep)
                    o = EcdcImport(
                        date_rep=date_rep,
                        day=row['day'],
                        month=row['month'],
                        year=row['year'],
                        cases=row['cases'],
                        deaths=row['deaths'],
                        countries_and_territories=row['countriesAndTerritories'],
                        geo_id=row['geoId'],
                        country_territory_code=row['countryterritoryCode'],
                        pop_data_2019=row['popData2019'],
                        continent_exp=row['continentExp'],
                        cumulative_number_for_14_days_of_covid19_cases_per_100000
                        =row['Cumulative_number_for_14_days_of_COVID-19_cases_per_100000'],
                        # datum=d.datum,
                        # year=d.year,
                        # month=d.month,
                        # day_of_month=d.day_of_month,
                        # day_of_week=d.day_of_week,
                        # week_of_year=d.day_of_week,
                        # year_week=d.year_week
                    )
                    db.session.add(o)
                    k = k + 1
                    if (k % 1000) == 0:
                        db.session.commit()
                        app.logger.info("  import ECDC  ...  " + str(k) + " rows")
            db.session.commit()
            app.logger.info("  import ECDC  ...  " + str(k) + " rows total")
        except KeyError as error:
            app.logger.warning("KeyError: import ECDC [begin]")
            app.logger.warning(":::" + str(error) + ":::")
            for item_key, item_value in row.items():
                app.logger.warning(item_key + " : " + item_value)
            app.logger.warning("KeyError: import ECDC [end]")
        except (Exception, psycopg2.DatabaseError) as error:
            app.logger.warning("WARN: import ECDC [begin]")
            app.logger.warning(error)
            app.logger.warning("WARN: import ECDC [end]")
        finally:
            app.logger.info("")
            app.logger.info("------------------------------------------------------------")
            app.logger.info(" imported into TABLE: "+self.cfg.tablename+" from "+self.cfg.cvsfile_path)
            app.logger.info("------------------------------------------------------------")
            app.logger.info(" import ECDC [done]")
        return self
