
from database import db, ITEMS_PER_PAGE
from flask_covid19.blueprints.app_all.all_model import BlueprintDateReported


class RkiVaccinationDateReported(BlueprintDateReported):
    __tablename__ = 'rki_vaccination_date_reported'
    __mapper_args__ = {
        'concrete': True
    }
    __table_args__ = (
        db.UniqueConstraint(
            'date_reported_import_str',
            'datum',
            'year_day_of_year',
            name="uix_rki_vaccination_date_reported"
        ),
    )

    id = db.Column(db.Integer, primary_key=True)
    datum = db.Column(db.Date, nullable=False, unique=True)
    year = db.Column(db.Integer, nullable=False)
    month = db.Column(db.Integer, nullable=False)
    day_of_month = db.Column(db.Integer, nullable=False)
    day_of_week = db.Column(db.Integer, nullable=False)
    week_of_year = db.Column(db.Integer, nullable=False)
    day_of_year = db.Column(db.Integer, nullable=True)
    year_week = db.Column(db.String(255), nullable=False)
    year_day_of_year = db.Column(db.String(255), nullable=False, unique=True)
    date_reported_import_str = db.Column(db.String(255), nullable=False, unique=True)
    processed = db.Column(db.Boolean, nullable=False)

    @classmethod
    def create_new_object_factory(cls, my_date_rep):
        return super().create_new_object_factory_rki_vaccination(my_date_rep)


class RkiVaccinationData(db.Model):
    __tablename__ = 'rki_vaccination'

    id = db.Column(db.Integer, primary_key=True)
    date_reported_id = db.Column(db.Integer, db.ForeignKey('rki_vaccination_date_reported.id'), nullable=False)
    date_reported = db.relationship(
        'RkiVaccinationDateReported',
        lazy='joined',
        cascade='save-update',
        order_by='desc(RkiVaccinationDateReported.datum)')
    dosen_kumulativ = db.Column(db.Integer, nullable=False)
    dosen_differenz_zum_vortag = db.Column(db.Integer, nullable=False)
    dosen_biontech_kumulativ = db.Column(db.Integer, nullable=False)
    dosen_moderna_kumulativ = db.Column(db.Integer, nullable=False)
    personen_erst_kumulativ = db.Column(db.Integer, nullable=False)
    personen_voll_kumulativ = db.Column(db.Integer, nullable=False)
    impf_quote_erst = db.Column(db.Float, nullable=False)
    impf_quote_voll = db.Column(db.Float, nullable=False)
    indikation_alter_dosen = db.Column(db.Integer, nullable=False)
    indikation_beruf_dosen = db.Column(db.Integer, nullable=False)
    indikation_medizinisch_dosen = db.Column(db.Integer, nullable=False)
    indikation_pflegeheim_dosen = db.Column(db.Integer, nullable=False)
    indikation_alter_erst = db.Column(db.Integer, nullable=False)
    indikation_beruf_erst = db.Column(db.Integer, nullable=False)
    indikation_medizinisch_erst = db.Column(db.Integer, nullable=False)
    indikation_pflegeheim_erst = db.Column(db.Integer, nullable=False)
    indikation_alter_voll = db.Column(db.Integer, nullable=False)
    indikation_beruf_voll = db.Column(db.Integer, nullable=False)
    indikation_medizinisch_voll = db.Column(db.Integer, nullable=False)
    indikation_pflegeheim_voll = db.Column(db.Integer, nullable=False)

    @classmethod
    def remove_all(cls):
        for one in cls.get_all():
            db.session.delete(one)
        db.session.commit()
        return None

    @classmethod
    def get_all_as_page(cls, page: int):
        return db.session.query(cls)\
            .paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_all(cls):
        return db.session.query(cls).all()

    @classmethod
    def get_by_id(cls, other_id: int):
        return db.session.query(cls)\
            .filter(cls.id == other_id)\
            .one()

    @classmethod
    def find_by_id(cls, other_id: int):
        return db.session.query(cls) \
            .filter(cls.id == other_id) \
            .one_or_none()

    @classmethod
    def find_by_datum(cls, other_datum: RkiVaccinationDateReported):
        return db.session.query(cls) \
            .filter(cls.date_reported_id == other_datum.id) \
            .one_or_none()
