import sys
import csv
import psycopg2
from datetime import date
from database import db, app
from flask_covid19.blueprints.app_all.all_config import BlueprintConfig
from flask_covid19.blueprints.data_who.who_model import WhoDateReported
from flask_covid19.blueprints.data_who.who_model_import import WhoImport, WhoFlat


class WhoServiceImport:
    def __init__(self, database, config: BlueprintConfig):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" WHO Service Import [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = config
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" WHO Service Import [ready]")

    def import_file(self):
        app.logger.info(" import WHO [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info(" import into TABLE: "+self.cfg.tablename+" from "+self.cfg.cvsfile_path)
        app.logger.info("------------------------------------------------------------")
        row = None
        if sys.platform == 'linux':
            keyDate_reported ='\ufeffDate_reported'
        else:
            keyDate_reported = 'ï»¿Date_reported'
        try:
            WhoImport.remove_all()
            with open(self.cfg.cvsfile_path, newline='\n') as csv_file:
                file_reader = csv.DictReader(csv_file, delimiter=',', quotechar='"')
                k = 0
                for row in file_reader:
                    date_reported = row[keyDate_reported]
                    # my_datum = date.fromisoformat(date_reported)
                    my_datum = WhoDateReported.create_new_object_factory(date_reported)
                    o = WhoImport(
                        new_cases=row['New_cases'],
                        cumulative_cases=row['Cumulative_cases'],
                        new_deaths=row['New_deaths'],
                        cumulative_deaths=row['Cumulative_deaths'],
                        country_code=row['Country_code'],
                        country=row['Country'],
                        who_region=row['WHO_region'],
                        date_reported=row[keyDate_reported],
                        datum=my_datum.datum
                    )
                    db.session.add(o)
                    oo = WhoFlat(
                        new_cases=row['New_cases'],
                        cumulative_cases=row['Cumulative_cases'],
                        new_deaths=row['New_deaths'],
                        cumulative_deaths=row['Cumulative_deaths'],
                        country_code=row['Country_code'],
                        country=row['Country'],
                        who_region=row['WHO_region'],
                        date_reported=row[keyDate_reported],
                        datum=my_datum.datum,
                        year=my_datum.year,
                        month=my_datum.month,
                        day_of_month=my_datum.day_of_month,
                        day_of_week=my_datum.day_of_month,
                        week_of_year=my_datum.week_of_year,
                        day_of_year=my_datum.day_of_year,
                        year_week=my_datum.year_week,
                        year_day_of_year=my_datum.year_day_of_year,
                        date_reported_import_str=date_reported,
                        location=row['Country'],
                        location_group=row['WHO_region']
                    )
                    db.session.add(oo)
                    k += 1
                    if (k % 2000) == 0:
                        db.session.commit()
                        app.logger.info(" import WHO  ... " + str(k) + " rows")
            db.session.commit()
            app.logger.info(" import WHO  ... " + str(k) + " rows total")
        except KeyError as error:
            app.logger.warning("WARN: import WHO [begin]")
            app.logger.warning(":::"+str(error)+":::")
            for item_key, item_value in row.items():
                app.logger.warning(item_key+" : "+item_value)
            app.logger.warning("WARN: import WHO [end]")
        except (Exception, psycopg2.DatabaseError) as error:
            app.logger.warning("WARN: import WHO [begin]")
            app.logger.warning(error)
            app.logger.warning("WARN: import WHO [end]")
        finally:
            app.logger.info("")
            app.logger.info("------------------------------------------------------------")
            app.logger.info(" imported into TABLE: "+self.cfg.tablename+" from "+self.cfg.cvsfile_path)
            app.logger.info("------------------------------------------------------------")
            app.logger.info(" import WHO [done]")
        return self
